@@ -15,13 +15,13 @@ from io import open
 from time import time
 from tqdm import tqdm
 from math import pow, tanh
-from .util import Print, VERBOSITY, Preproc as Pp
+from .util import is_a_collection, Print, VERBOSITY, Preproc as Pp
 
 # python 2 and 3 compatibility
 from functools import reduce
 from six.moves import xrange
 
-__version__ = "0.5.9"
+__version__ = "0.6.3"
 
 ENCODING = "utf-8"
 
@@ -31,6 +31,7 @@ WORD_DELTR = r"\s"
 WORD_REGEX = r"\w+(?:'\w+)?"
 
 STR_UNKNOWN, STR_MOST_PROBABLE = "unknown", "most-probable"
+STR_OTHERS_CATEGORY = "[others]"
 STR_UNKNOWN_CATEGORY = "[unknown]"
 IDX_UNKNOWN_CATEGORY = -1
 STR_UNKNOWN_WORD = ''
@@ -82,9 +83,9 @@ class SS3:
                  term (word or n-grams), options are:
                  "norm_gv_xai", "norm_gv" and "gv" (default: "norm_gv_xai")
     :type cv_m: str
-    :param sn_m: method used to compute the sanction (sn) function, options
+    :param sg_m: method used to compute the significance (sg) function, options
                  are: "vanilla" and "xai" (default: "xai")
-    :type sn_m: str
+    :type sg_m: str
     """
 
     __name__ = "model"
@@ -95,6 +96,8 @@ class SS3:
     __p__ = 1
     __a__ = .0
 
+    __multilabel__ = False
+
     __l_update__ = None
     __s_update__ = None
     __p_update__ = None
@@ -116,7 +119,7 @@ class SS3:
 
     def __init__(
         self, s=None, l=None, p=None, a=None,
-        name="", cv_m=STR_NORM_GV_XAI, sn_m=STR_XAI
+        name="", cv_m=STR_NORM_GV_XAI, sg_m=STR_XAI
     ):
         """
         Class constructor.
@@ -137,16 +140,22 @@ class SS3:
                      term (word or n-grams), options are:
                      "norm_gv_xai", "norm_gv" and "gv" (default: "norm_gv_xai")
         :type cv_m: str
-        :param sn_m: method used to compute the sanction (sn) function, options
+        :param sg_m: method used to compute the significance (sg) function, options
                      are: "vanilla" and "xai" (default: "xai")
-        :type sn_m: str
+        :type sg_m: str
+        :raises: ValueError
         """
         self.__name__ = (name or self.__name__).lower()
 
-        self.__s__ = s or self.__s__
-        self.__l__ = l or self.__l__
-        self.__p__ = p or self.__p__
-        self.__a__ = a or self.__a__
+        self.__s__ = self.__s__ if s is None else s
+        self.__l__ = self.__l__ if l is None else l
+        self.__p__ = self.__p__ if p is None else p
+        self.__a__ = self.__a__ if a is None else a
+
+        try:
+            float(self.__s__ + self.__l__ + self.__p__ + self.__a__)
+        except BaseException:
+            raise ValueError("hyperparameter values must be numbers")
 
         self.__categories_index__ = {}
         self.__categories__ = []
@@ -163,13 +172,13 @@ class SS3:
         elif cv_m == STR_GV:
             self.__cv__ = self.__gv__
 
-        if sn_m == STR_XAI:
+        if sg_m == STR_XAI:
             self.__sg__ = self.__sg_xai__
-        elif sn_m == STR_VANILLA:
+        elif sg_m == STR_VANILLA:
             self.__sg__ = self.__sg_vanilla__
 
         self.__cv_mode__ = cv_m
-        self.__sn_mode__ = sn_m
+        self.__sg_mode__ = sg_m
 
         self.original_sumop_ngrams = self.summary_op_ngrams
         self.original_sumop_sentences = self.summary_op_sentences
@@ -205,7 +214,10 @@ class SS3:
 
         s = sum([min(v, 1) for v in m_values])
 
-        return pow((c - (s + 1)) / ((c - 1) * (s + 1)), self.__p__)
+        try:
+            return pow((c - (s + 1)) / ((c - 1) * (s + 1)), self.__p__)
+        except ZeroDivisionError:  # if c <= 1
+            return 1.
 
     def __sg_vanilla__(self, ngram, icat, cache=True):
         """The original significance (sg) function definition."""
@@ -553,8 +565,6 @@ class SS3:
 
         If category name doesn't exist, creates a new one.
         """
-        if isinstance(name, str):
-            name = name.lower()
         try:
             return self.__categories_index__[name]
         except KeyError:
@@ -626,6 +636,17 @@ class SS3:
         )
         return sorted(vocab_icat, key=lambda k: -k[-1])
 
+    def __get_def_cat__(self, def_cat):
+        """Given the `def_cat` argument, get the default category value."""
+        if def_cat is not None and (def_cat not in [STR_MOST_PROBABLE, STR_UNKNOWN] and
+                                    self.get_category_index(def_cat) == IDX_UNKNOWN_CATEGORY):
+            raise ValueError(
+                "the default category must be 'most-probable', 'unknown', or a category name"
+                " (current value is '%s')." % str(def_cat)
+            )
+        def_cat = None if def_cat == STR_UNKNOWN else def_cat
+        return self.get_most_probable_category() if def_cat == STR_MOST_PROBABLE else def_cat
+
     def __get_next_iwords__(self, sent, icat):
         """Return the list of possible following words' indexes."""
         if not self.get_category_name(icat):
@@ -794,6 +815,9 @@ class SS3:
             if def_cat == IDX_UNKNOWN_CATEGORY:
                 raise InvalidCategoryError
 
+        # does the special "[others]" category exist? (only used in multilabel classification)
+        __other_idx__ = self.get_category_index(STR_OTHERS_CATEGORY)
+
         if self.__update_needed__():
             self.update_values()
 
@@ -847,24 +871,29 @@ class SS3:
             else:
                 if pred_cv.sum() == 0:
                     if def_cat == IDX_UNKNOWN_CATEGORY:
-                        y_pred[doc_idx] = [STR_UNKNOWN_CATEGORY if labels else def_cat]
+                        y_pred[doc_idx] = []
                     else:
                         y_pred[doc_idx] = [self.get_category_name(def_cat) if labels else def_cat]
                 else:
-                    r = sorted(
-                        [
-                            (i, pred_cv[i])
-                            for i in range(pred_cv.size)
-                        ],
-                        key=lambda e: -e[1]
-                    )
+                    r = sorted([(i, pred_cv[i])
+                                for i in range(pred_cv.size)],
+                               key=lambda e: -e[1])
                     if labels:
-                        y_pred[doc_idx] = [
-                            self.get_category_name(cat_i)
-                            for cat_i, _ in r[:kmean_multilabel_size(r)]
-                        ]
+                        y_pred[doc_idx] = [self.get_category_name(cat_i)
+                                           for cat_i, _ in r[:kmean_multilabel_size(r)]]
                     else:
                         y_pred[doc_idx] = [cat_i for cat_i, _ in r[:kmean_multilabel_size(r)]]
+
+                # if the special "[others]" category exists
+                if __other_idx__ != IDX_UNKNOWN_CATEGORY:
+                    # if its among the predicted labels, remove (hide) it
+                    if labels:
+                        if STR_OTHERS_CATEGORY in y_pred[doc_idx]:
+                            y_pred[doc_idx].remove(STR_OTHERS_CATEGORY)
+                    else:
+                        if __other_idx__ in y_pred[doc_idx]:
+                            y_pred[doc_idx].remove(__other_idx__)
+
         return y_pred
 
     def summary_op_ngrams(self, cvs):
@@ -1169,7 +1198,7 @@ class SS3:
         """
         return self.__a__
 
-    def get_categories(self):
+    def get_categories(self, all=False):
         """
         Get the list of category names.
 
@@ -1179,6 +1208,7 @@ class SS3:
         return [
             self.get_category_name(ci)
             for ci in range(len(self.__categories__))
+            if all or self.get_category_name(ci) != STR_OTHERS_CATEGORY
         ]
 
     def get_most_probable_category(self):
@@ -1210,8 +1240,6 @@ class SS3:
         :rtype: int
         """
         try:
-            if isinstance(name, str):
-                name = name.lower()
             return self.__categories_index__[name]
         except KeyError:
             return IDX_UNKNOWN_CATEGORY
@@ -1372,7 +1400,8 @@ class SS3:
             "__index_to_word__": self.__index_to_word__,
             "__word_to_index__": self.__word_to_index__,
             "__cv_mode__": self.__cv_mode__,
-            "__sn_mode__": self.__sn_mode__
+            "__sg_mode__": self.__sg_mode__,
+            "__multilabel__": self.__multilabel__
         }
 
         try:
@@ -1427,21 +1456,24 @@ class SS3:
                 STR_MODEL_EXT
             ), "r", encoding=ENCODING
         )
-        json_file_format = json.loads(json_file.read(), object_hook=key_as_int)
+        jmodel = json.loads(json_file.read(), object_hook=key_as_int)
         json_file.close()
 
-        self.__max_fr__ = json_file_format["__max_fr__"]
-        self.__max_gv__ = json_file_format["__max_gv__"]
-        self.__l__ = json_file_format["__l__"]
-        self.__p__ = json_file_format["__p__"]
-        self.__s__ = json_file_format["__s__"]
-        self.__a__ = json_file_format["__a__"]
-        self.__categories__ = json_file_format["__categories__"]
-        self.__categories_index__ = json_file_format["__categories_index__"]
-        self.__index_to_word__ = json_file_format["__index_to_word__"]
-        self.__word_to_index__ = json_file_format["__word_to_index__"]
-        self.__cv_mode__ = json_file_format["__cv_mode__"]
-        self.__sn_mode__ = json_file_format["__sn_mode__"]
+        self.__max_fr__ = jmodel["__max_fr__"]
+        self.__max_gv__ = jmodel["__max_gv__"]
+        self.__l__ = jmodel["__l__"]
+        self.__p__ = jmodel["__p__"]
+        self.__s__ = jmodel["__s__"]
+        self.__a__ = jmodel["__a__"]
+        self.__categories__ = jmodel["__categories__"]
+        self.__categories_index__ = jmodel["__categories_index__"]
+        self.__index_to_word__ = jmodel["__index_to_word__"]
+        self.__word_to_index__ = jmodel["__word_to_index__"]
+        self.__cv_mode__ = jmodel["__cv_mode__"]
+        self.__multilabel__ = jmodel["__multilabel__"] if "__multilabel__" in jmodel else False
+        self.__sg_mode__ = (jmodel["__sg_mode__"]
+                            if "__sg_mode__" in jmodel
+                            else jmodel["__sn_mode__"])
 
         self.__zero_cv__ = (0,) * len(self.__categories__)
         self.__s_update__ = self.__s__
@@ -1909,6 +1941,9 @@ class SS3:
         """
         self.__cv_cache__ = None
 
+        if not doc or cat is None:
+            return
+
         try:
             doc = doc.decode(ENCODING)
         except UnicodeEncodeError:  # for python 2 compatibility
@@ -2121,7 +2156,7 @@ class SS3:
 
         return cat if labels else self.get_category_index(cat)
 
-    def classify_multilabel(self, doc, def_cat=STR_MOST_PROBABLE, labels=True, prep=True):
+    def classify_multilabel(self, doc, def_cat=STR_UNKNOWN, labels=True, prep=True):
         """
         Classify a given document returning multiple category labels.
 
@@ -2134,7 +2169,7 @@ class SS3:
         :param def_cat: default category to be assigned when SS3 is not
                         able to classify a document. Options are
                         "most-probable", "unknown" or a given category name.
-                        (default: "most-probable")
+                        (default: "unknown")
         :type def_cat: str
         :param labels: whether to return the category labels or just the
                        category indexes (default: True)
@@ -2149,22 +2184,31 @@ class SS3:
 
         if not r or not r[0][1]:
             if not def_cat or def_cat == STR_UNKNOWN:
-                cat = STR_UNKNOWN_CATEGORY
+                return []
             elif def_cat == STR_MOST_PROBABLE:
                 cat = self.get_most_probable_category()
             else:
                 if self.get_category_index(def_cat) == IDX_UNKNOWN_CATEGORY:
                     raise InvalidCategoryError
                 cat = def_cat
-            return [cat] if labels else [self.get_category_index(cat)]
-        else:
-            if labels:
-                return [
-                    self.get_category_name(cat_i)
-                    for cat_i, _ in r[:kmean_multilabel_size(r)]
-                ]
+            if cat != STR_OTHERS_CATEGORY:
+                return [cat] if labels else [self.get_category_index(cat)]
             else:
-                return [cat_i for cat_i, _ in r[:kmean_multilabel_size(r)]]
+                return []
+        else:
+            __other_idx__ = self.get_category_index(STR_OTHERS_CATEGORY)
+            if labels:
+                result = [self.get_category_name(cat_i)
+                          for cat_i, _ in r[:kmean_multilabel_size(r)]]
+                # removing "hidden" special category ("[other]")
+                if __other_idx__ != IDX_UNKNOWN_CATEGORY and STR_OTHERS_CATEGORY in result:
+                    result.remove(STR_OTHERS_CATEGORY)
+            else:
+                result = [cat_i for cat_i, _ in r[:kmean_multilabel_size(r)]]
+                # removing "hidden" special category ("[other]")
+                if __other_idx__ != IDX_UNKNOWN_CATEGORY and __other_idx__ in result:
+                    result.remove(__other_idx__)
+            return result
 
     def fit(self, x_train, y_train, n_grams=1, prep=True, leave_pbar=True):
         """
@@ -2173,7 +2217,8 @@ class SS3:
         :param x_train: the list of documents
         :type x_train: list (of str)
         :param y_train: the list of document labels
-        :type y_train: list (of str)
+        :type y_train: list of str for singlelabel classification;
+                        list of list of str for multilabel classification.
         :param n_grams:  indicates the maximum ``n``-grams to be learned
                         (e.g. a value of ``1`` means only 1-grams (words),
                         ``2`` means 1-grams and 2-grams,
@@ -2184,36 +2229,65 @@ class SS3:
         :param leave_pbar: controls whether to leave the progress bar or
                            remove it after finishing.
         :type leave_pbar: bool
+        :raises: ValueError
         """
-        cats = sorted(list(set(y_train)))
         stime = time()
-
         x_train, y_train = list(x_train), list(y_train)
 
-        x_train = [
-            "".join([
-                x_train[i]
-                if x_train[i] and x_train[i][-1] == '\n'
-                else
-                x_train[i] + '\n'
-                for i in xrange(len(x_train))
-                if y_train[i] == cat
-            ])
-            for cat in cats
-        ]
-        y_train = list(cats)
+        if len(x_train) != len(y_train):
+            raise ValueError("`x_train` and `y_train` must have the same length")
+
+        if len(y_train) == 0:
+            raise ValueError("`x_train` and `y_train` are empty")
+
+        # if it's a multi-label classification problem
+        if is_a_collection(y_train[0]):
+            # flattening y_train
+            labels = [l for y in y_train for l in y]
+            self.__multilabel__ = True
+        else:
+            labels = y_train
+
+        cats = sorted(list(set(labels)))
+
+        # if it's a single-label classification problem
+        if not is_a_collection(y_train[0]):
+            x_train = [
+                "".join([
+                    x_train[i]
+                    if x_train[i] and x_train[i][-1] == '\n'
+                    else
+                    x_train[i] + '\n'
+                    for i in xrange(len(x_train))
+                    if y_train[i] == cat
+                ])
+                for cat in cats
+            ]
+            y_train = list(cats)
 
         Print.info("about to start training", offset=1)
         Print.verbosity_region_begin(VERBOSITY.NORMAL)
         progress_bar = tqdm(total=len(x_train), desc="Training",
                             leave=leave_pbar, disable=Print.is_quiet())
-        for i in range(len(x_train)):
-            progress_bar.set_description_str("Training on '%s'" % str(y_train[i]))
-            self.learn(
-                x_train[i], y_train[i],
-                n_grams=n_grams, prep=prep, update=False
-            )
-            progress_bar.update(1)
+
+        # if it's a multi-label classification problem
+        if is_a_collection(y_train[0]):
+            __others__ = [STR_OTHERS_CATEGORY]
+            for i in range(len(x_train)):
+                for label in (y_train[i] if y_train[i] else __others__):
+                    self.learn(
+                        x_train[i], label,
+                        n_grams=n_grams, prep=prep, update=False
+                    )
+                progress_bar.update(1)
+        else:
+            for i in range(len(x_train)):
+                progress_bar.set_description_str("Training on '%s'" % str(y_train[i]))
+                self.learn(
+                    x_train[i], y_train[i],
+                    n_grams=n_grams, prep=prep, update=False
+                )
+                progress_bar.update(1)
         progress_bar.close()
         self.__prune_tries__()
         Print.verbosity_region_end()
@@ -2250,7 +2324,7 @@ class SS3:
         ]
 
     def predict(
-        self, x_test, def_cat=STR_MOST_PROBABLE,
+        self, x_test, def_cat=None,
         labels=True, multilabel=False, prep=True, leave_pbar=True
     ):
         """
@@ -2261,6 +2335,8 @@ class SS3:
         :param def_cat: default category to be assigned when SS3 is not
                         able to classify a document. Options are
                         "most-probable", "unknown" or a given category name.
+                        (default: "most-probable", or "unknown" for
+                         multi-label classification)
         :type def_cat: str
         :param labels: whether to return the list of category names or just
                        category indexes
@@ -2268,6 +2344,8 @@ class SS3:
         :param multilabel: whether to perform multi-label classification or not.
                            if enabled, for each document returns a ``list`` of labels
                            instead of a single label (``str``).
+                           If the model was trained using multilabeled data, then this
+                           argument will be ignored and set to True.
         :type multilabel: bool
         :param prep: enables the default input preprocessing (default: True)
         :type prep: bool
@@ -2282,11 +2360,17 @@ class SS3:
         if not self.__categories__:
             raise EmptyModelError
 
+        multilabel = multilabel or self.__multilabel__
+
+        if def_cat is None:
+            def_cat = STR_UNKNOWN if multilabel else STR_MOST_PROBABLE
+
         if not def_cat or def_cat == STR_UNKNOWN:
-            Print.info(
-                "default category was set to 'unknown' (its index will be -1)",
-                offset=1
-            )
+            if not multilabel:
+                Print.info(
+                    "default category was set to 'unknown' (its index will be -1)",
+                    offset=1
+                )
         else:
             if def_cat == STR_MOST_PROBABLE:
                 Print.info(
@@ -2480,6 +2564,10 @@ def kmean_multilabel_size(res):
     clust = {"neg": [], "pos": []}  # clusters (2 clusters: "pos" and "neg")
     new_cent_neg = res[-1][1]
     new_cent_pos = res[0][1]
+
+    if new_cent_neg == new_cent_pos:
+        return 0
+
     while (cent["pos"] != new_cent_pos) or (cent["neg"] != new_cent_neg):
         cent["neg"], cent["pos"] = new_cent_neg, new_cent_pos
         clust["neg"], clust["pos"] = [], []
